/*
 * @(#)HorizontalLineRemovalSettingsDialogBox.java	1.0 08/15/04
 *
 * McGill Univarsity
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * A dialog box that allows the user to select thresholds that control which
 * pixels are to be selected as belonging to horizontal lines of some specified
 * colour or combination of colour.
 * 
 * <p>Parameters are chosen to set the RGB colour channel intervals that candidate pixels
 * that could belong to the horizontal lines must fall within (including absolute 8 bit
 * values, fraction of total level per channel and average brightness). The user may also
 * choose to set thresholds based on maximum vertical run length and minimum horizontal run
 * length. The <i>Filter Pixels By</i> checkboxes enable the user to control which of these
 * options are used, and the text boxes allow the users to set the parameters. The filtering is
 * performed in the same order that the <i>Filter Pixels By</i> options are listed, with
 * only the pixels surviving one stage serving as candidates to the next.
 *
 * <p>Users can use the <i>Test Thresholds</i> button to restore the image to that of
 * the file that was  most recently loaded or saved and then cause those pixels that
 * survive various thresholds to be superimposed on the image. Only the last <i>n</i>
 * layers, as set by the <i>Max Layers To Display</i> text area are displayed. The pixels
 * surviving the last stage of filtering are set to bright green, those surviving the
 * second to last stage are set to bright orange, those surviving the third to last stage 
 * are set to blue, those survivng the fourth to last stage are set to magenta and those
 * surviving the first stage are set to magenta. The <i>Restore Defaults</i> button restores
 * the values of the settings to their hard coded values.
 *
 * <p>The <i>Done</i> button hides this dialogbox.
 *
 * <p>The following parameters can be set:
 *
 * <p><b>Min Absolute Red:</b> the minimum red RGB channel level that a
 * pixel must have in order to be selected as a candidate. Value must be between 0
 * and 255 and must be lower than the <i>Max Absolute Red</i>.
 * <br><b>Max Absolute Red:</b> the maximum red RGB channel level that a
 * pixel must have in order to be selected as a candidate. Value must be between 0
 * and 255 and must be higher than the <i>Min Absolute Red</i>.
 * <br><b>Min Absolute Green:</b> the minimum green RGB channel level that a
 * pixel must have in order to be selected as a candidate. Value must be between 0
 * and 255 and must be lower than the <i>Max Absolute Green</i>.
 * <br><b>Max Absolute Green:</b> the maximum green RGB channel level that a
 * pixel must have in order to be selected as a candidate. Value must be between 0
 * and 255 and must be higher than the <i>Min Absolute Green</i>.
 * <br><b>Min Absolute Blue:</b> the minimum blue RGB channel level that a
 * pixel must have in order to be selected as a candidate. Value must be between 0
 * and 255 and must be lower than the <i>Max Absolute Blue</i>.
 * <br><b>Max Absolute Blue:</b> the maximum blue RGB channel level that a
 * pixel must have in order to be selected as a candidate. Value must be between 0
 * and 255 and must be higher than the <i>Min Absolute Blue</i>.
 * <br><b>Min Red Percentage:</b> the minimum percentage of the total RGB levels
 * represented by the red channel level that a pixel must have in order to be
 * selected as a candidate. Value must be between 0.0 and 100.0 and must be lower
 * than the <i>Red Blue Percentage</i>.
 * <br><b>Max Red Percentage:</b> the maximum percentage of the total RGB levels
 * represented by the red channel level that a pixel must have in order to be
 * selected as a candidate. Value must be between 0.0 and 100.0 and must be higher
 * than the <i>Min Red Percentage</i>.
 * <br><b>Min Green Percentage:</b> the minimum percentage of the total RGB levels
 * represented by the green channel level that a pixel must have in order to be
 * selected as a candidate. Value must be between 0.0 and 100.0 and must be lower
 * than the <i>Green Blue Percentage</i>.
 * <br><b>Max Green Percentage:</b> the maximum percentage of the total RGB levels
 * represented by the green channel level that a pixel must have in order to be
 * selected as a candidate. Value must be between 0.0 and 100.0 and must be higher
 * than the <i>Min Green Percentage</i>.
 * <br><b>Min Blue Percentage:</b> the minimum percentage of the total RGB levels
 * represented by the blue channel level that a pixel must have in order to be
 * selected as a candidate. Value must be between 0.0 and 100.0 and must be lower
 * than the <i>Max Blue Percentage</i>.
 * <br><b>Max Blue Percentage:</b> the maximum percentage of the total RGB levels
 * represented by the blue channel level that a pixel must have in order to be
 * selected as a candidate. Value must be between 0.0 and 100.0 and must be higher
 * than the <i>Min Blue Percentage</i>.
 * <br><b>Min Brightness:</b> the minimum average level of the three RGB channels
 * that a pixel must have in order to be selected as a candidate. Value must be 
 * between 0.0 and 255.0 and must be lower than the <i>Max Brightness</i>.
 * <br><b>Max Brightness:</b> the maximum average level of the three RGB channels
 * that a pixel may have in order to be selected as a candidate. Value must be 
 * between 0.0 and 255.0 and must be higher than the <i>Min Brightness</i>.
 * <br><b>Max Vertical Run Length:</b> the maximum number of consecutive vertical
 * pixels selected by the previous threshold filtering that are acceptable in order
 * for pixels to be considered part of a horizontal line.
 * <br><b>Min Horizontal Run Length:</b> the minimum number of consecutive horizontal
 * pixels selected by the previous threshold filtering that are acceptable in order
 * for pixels to be considered part of a horizontal line.
 * <br><b>Filter Pixels By:</b> which types of threshold filtering to apply.
 * <br><b>Max Layers To Display:</b> the maximum number of filtering layers that
 * are to be displayed when the <i>Test Threshold</i> button is pressed. If more
 * than this number of layers are selected with the <i>Filter Pixels By</i>
 * checkbox, then only the last one(s) are displayed.
 *
 * <p>These parameters can be accessed by other objects through this class' get
 * methods.
 *
 * @author Cory McKay
 */ 
public class HorizontalLineRemovalSettingsDialogBox
	extends JFrame
	implements ActionListener
{
	/* FIELDS ******************************************************************/


	// Window that instantiated this window
	private MainFrame	parent_window;
	
	// Containers to hold Swing elements
	private Container		content_pane;
	private JPanel			settings_panel;
	private JPanel			button_panel;
	
	// Threshold parameter editors
	private JTextArea		red_detection_low_threshold_text_area;
	private JTextArea		red_detection_high_threshold_text_area;
	private JTextArea		green_detection_low_threshold_text_area;
	private JTextArea		green_detection_high_threshold_text_area;
	private JTextArea		blue_detection_low_threshold_text_area;
	private JTextArea		blue_detection_high_threshold_text_area;
	private JTextArea		red_min_percentage_threshold_text_area;
	private JTextArea		red_max_percentage_threshold_text_area;
	private JTextArea		green_min_percentage_threshold_text_area;
	private JTextArea		green_max_percentage_threshold_text_area;
	private JTextArea		blue_min_percentage_threshold_text_area;
	private JTextArea		blue_max_percentage_threshold_text_area;
	private JTextArea		low_brightness_threshold_text_area;
	private JTextArea		high_brightness_threshold_text_area;
	private JTextArea		vertical_run_length_threshold_text_area;
	private JTextArea		horizontal_run_length_threshold_text_area;
	private JTextArea		number_layers_to_display_text_area;

	// Check boxes
	private	JCheckBox		filter_by_colour_absolute_values_checkbox;
	private	JCheckBox		filter_by_colour_fractions_checkbox;
	private	JCheckBox		filter_by_brightness_checkbox;
	private	JCheckBox		filter_by_vertical_run_length_checkbox;
	private	JCheckBox		filter_by_horizontal_run_length_checkbox;

	// Buttons
	private JButton			done_button;
	private JButton			restore_defaults_button;
	private JButton			test_thresholds_button;


	/* CONSTRUCTOR *************************************************************/


	/**
	 * Basic constructor. Configures the panel and sets its fields to defaults.
	 *
 	 * @param	parent	Name of the <code>MainFrame</code> window that spawned this dialog box.
	 */
	public HorizontalLineRemovalSettingsDialogBox(MainFrame parent)
	{
		// Store parent recording
		parent_window = parent;

		// Configure overall window settings
		setTitle("Horizontal Line Removal Settings");
		int horizontal_gap = 6; // horizontal space between GUI elements
		int vertical_gap = 11; // horizontal space between GUI elements
		content_pane = getContentPane();
		content_pane.setLayout(new BorderLayout(horizontal_gap, vertical_gap));
		settings_panel = new JPanel(new GridLayout(25, 2, horizontal_gap, vertical_gap));
		button_panel = new JPanel(new FlowLayout());

		// Configure parameter elements
		int number_columns = 7;
		red_detection_low_threshold_text_area = new JTextArea(1, number_columns);
		red_detection_high_threshold_text_area = new  JTextArea(1, number_columns);
		green_detection_low_threshold_text_area = new JTextArea(1, number_columns);
		green_detection_high_threshold_text_area = new  JTextArea(1, number_columns);
		blue_detection_low_threshold_text_area = new JTextArea(1, number_columns);
		blue_detection_high_threshold_text_area = new  JTextArea(1, number_columns);
		red_min_percentage_threshold_text_area = new  JTextArea(1, number_columns);
		red_max_percentage_threshold_text_area = new  JTextArea(1, number_columns);
		green_min_percentage_threshold_text_area = new  JTextArea(1, number_columns);
		green_max_percentage_threshold_text_area = new  JTextArea(1, number_columns);
		blue_min_percentage_threshold_text_area = new  JTextArea(1, number_columns);
		blue_max_percentage_threshold_text_area = new  JTextArea(1, number_columns);
		low_brightness_threshold_text_area = new JTextArea(1, number_columns);
		high_brightness_threshold_text_area = new JTextArea(1, number_columns);
		vertical_run_length_threshold_text_area = new  JTextArea(1, number_columns);
		horizontal_run_length_threshold_text_area = new  JTextArea(1, number_columns);
		number_layers_to_display_text_area = new JTextArea(1, number_columns);

		// Configure check boxes
		filter_by_colour_absolute_values_checkbox = new JCheckBox("Colour Absolute Values");
		filter_by_colour_fractions_checkbox = new JCheckBox("Colour Percentages");
		filter_by_brightness_checkbox = new JCheckBox("Brightness");
		filter_by_vertical_run_length_checkbox = new JCheckBox("Vertical Run Length");
		filter_by_horizontal_run_length_checkbox = new JCheckBox("Horizontal Run Length");

		// Set text areas and check boxes to default values
		restoreDefaultValues();
		
		// Configure buttons
		done_button = new JButton("Done");
		done_button.addActionListener(this);
		button_panel.add(done_button);
		restore_defaults_button = new JButton("Restore Defaults");
		restore_defaults_button.addActionListener(this);
		button_panel.add(restore_defaults_button);
		test_thresholds_button = new JButton("Test Thresholds");
		test_thresholds_button.addActionListener(this);
		button_panel.add(test_thresholds_button);

		// Add l elements to settings_panel
		settings_panel.add(new JLabel("Min Absolute Red (0-255):"));
		settings_panel.add(red_detection_low_threshold_text_area);
		settings_panel.add(new JLabel("Max Absolute Red (0-255):"));
		settings_panel.add(red_detection_high_threshold_text_area);
		settings_panel.add(new JLabel("Min Absolute Green (0-255):"));
		settings_panel.add(green_detection_low_threshold_text_area);
		settings_panel.add(new JLabel("Max Absolute Green (0-255):"));
		settings_panel.add(green_detection_high_threshold_text_area);
		settings_panel.add(new JLabel("Min Absolute Blue (0-255):"));
		settings_panel.add(blue_detection_low_threshold_text_area);
		settings_panel.add(new JLabel("Max Absolute Blue (0-255):"));
		settings_panel.add(blue_detection_high_threshold_text_area);
		settings_panel.add(new JLabel(""));
		settings_panel.add(new JLabel(""));
		settings_panel.add(new JLabel("Min Red Percentage (0-100):"));
		settings_panel.add(red_min_percentage_threshold_text_area);
		settings_panel.add(new JLabel("Max Red Percentage (0-100):"));
		settings_panel.add(red_max_percentage_threshold_text_area);
		settings_panel.add(new JLabel("Min Green Percentage (0-100):"));
		settings_panel.add(green_min_percentage_threshold_text_area);
		settings_panel.add(new JLabel("Max Green Percentage (0-100):"));
		settings_panel.add(green_max_percentage_threshold_text_area);
		settings_panel.add(new JLabel("Min Blue Percentage (0-100):"));
		settings_panel.add(blue_min_percentage_threshold_text_area);
		settings_panel.add(new JLabel("Max Blue Percentage (0-100):"));
		settings_panel.add(blue_max_percentage_threshold_text_area);
		settings_panel.add(new JLabel(""));
		settings_panel.add(new JLabel(""));
		settings_panel.add(new JLabel("Min Brightness (0-255):"));
		settings_panel.add(low_brightness_threshold_text_area);
		settings_panel.add(new JLabel("Max Brightness (0-255):"));
		settings_panel.add(high_brightness_threshold_text_area);
		settings_panel.add(new JLabel("Max Vertical Run Length:"));
		settings_panel.add(vertical_run_length_threshold_text_area);
		settings_panel.add(new JLabel("Min Horizontal Run Length:"));
		settings_panel.add(horizontal_run_length_threshold_text_area);
		settings_panel.add(new JLabel(""));
		settings_panel.add(new JLabel(""));
		settings_panel.add(new JLabel("Filter Pixels By:"));
		settings_panel.add(filter_by_colour_absolute_values_checkbox);
		settings_panel.add(new JLabel(""));
		settings_panel.add(filter_by_colour_fractions_checkbox);
		settings_panel.add(new JLabel(""));
		settings_panel.add(filter_by_brightness_checkbox);
		settings_panel.add(new JLabel(""));
		settings_panel.add(filter_by_vertical_run_length_checkbox);
		settings_panel.add(new JLabel(""));
		settings_panel.add(filter_by_horizontal_run_length_checkbox);
		settings_panel.add(new JLabel("Max Layers To Display (1-5):"));
		settings_panel.add(number_layers_to_display_text_area);

		// Add elements to the dialog box
		content_pane.add(settings_panel, BorderLayout.CENTER);
		content_pane.add(button_panel, BorderLayout.SOUTH);

		// Cause reaction when the exit box is pressed
		addWindowListener(new WindowAdapter()
		{
			public void windowClosing (WindowEvent e) { done(); }
		});

		// Prepare GUI for display
		pack();
	}


	/* PUBLIC METHODS **********************************************************/

	
	/**
	 * Calls the appropriate methods when the buttons are pressed.
	 *
	 * @param	event		The event that is to be reacted to.
	 */
	public void actionPerformed(ActionEvent event)
	{
		// React to the cancel_button
		if (event.getSource().equals(done_button))
			done();

		// React to the restore_defaults_button
		else if (event.getSource().equals(restore_defaults_button))
			restoreDefaultValues();

		// React to the test_thresholds_button
		else if (event.getSource().equals(test_thresholds_button))
			parent_window.showSelectedPixels();
	}

	
	/**
	 * Returns the value of low red detection threshold, as entered in the text field.
	 */
	public int getLowRedThreshold()
	{
		return Integer.parseInt(red_detection_low_threshold_text_area.getText());
	}


	/**
	 * Returns the value of high red detection threshold, as entered in the text field.
	 */
	public int getHighRedThreshold()
	{
		return Integer.parseInt(red_detection_high_threshold_text_area.getText());
	}


	/**
	 * Returns the value of low green detection threshold, as entered in the text field.
	 */
	public int getLowGreenThreshold()
	{
		return Integer.parseInt(green_detection_low_threshold_text_area.getText());
	}


	/**
	 * Returns the value of high green detection threshold, as entered in the text field.
	 */
	public int getHighGreenThreshold()
	{
		return Integer.parseInt(green_detection_high_threshold_text_area.getText());
	}


	/**
	 * Returns the value of low blue detection threshold, as entered in the text field.
	 */
	public int getLowBlueThreshold()
	{
		return Integer.parseInt(blue_detection_low_threshold_text_area.getText());
	}


	/**
	 * Returns the value of high blue detection threshold, as entered in the text field.
	 */
	public int getHighBlueThreshold()
	{
		return Integer.parseInt(blue_detection_high_threshold_text_area.getText());
	}


	/**
	 * Returns the value of minimum red percentage threshold, as entered in the text field.
	 */
	public double getMinRedPercentageThreshold()
	{
		return Double.parseDouble(red_min_percentage_threshold_text_area.getText());
	}


	/**
	 * Returns the value of maximum red percentage threshold, as entered in the text field.
	 */
	public double getMaxRedPercentageThreshold()
	{
		return Double.parseDouble(red_max_percentage_threshold_text_area.getText());
	}


	/**
	 * Returns the value of minimum green percentage threshold, as entered in the text field.
	 */
	public double getMinGreenPercentageThreshold()
	{
		return Double.parseDouble(green_min_percentage_threshold_text_area.getText());
	}


	/**
	 * Returns the value of maximum green percentage threshold, as entered in the text field.
	 */
	public double getMaxGreenPercentageThreshold()
	{
		return Double.parseDouble(green_max_percentage_threshold_text_area.getText());
	}


	/**
	 * Returns the value of minimum blue percentage threshold, as entered in the text field.
	 */
	public double getMinBluePercentageThreshold()
	{
		return Double.parseDouble(blue_min_percentage_threshold_text_area.getText());
	}


	/**
	 * Returns the value of maximum blue percentage threshold, as entered in the text field.
	 */
	public double getMaxBluePercentageThreshold()
	{
		return Double.parseDouble(blue_max_percentage_threshold_text_area.getText());
	}


	/**
	 * Returns the value of minimum brightness threshold, as entered in the text field.
	 */
	public double getMinBrightnessThreshold()
	{
		return Double.parseDouble(low_brightness_threshold_text_area.getText());
	}


	/**
	 * Returns the value of maximum brightness threshold, as entered in the text field.
	 */
	public double getMaxBrightnessThreshold()
	{
		return Double.parseDouble(high_brightness_threshold_text_area.getText());
	}


	/**
	 * Returns the value of vertical run length threshold, as entered in the text field.
	 */
	public int getVerticalRunLengthThreshold()
	{
		return Integer.parseInt(vertical_run_length_threshold_text_area.getText());
	}


	/**
	 * Returns the value of horizontal run length threshold, as entered in the text field.
	 */
	public int getHorizontalRunLengthThreshold()
	{
		return Integer.parseInt(horizontal_run_length_threshold_text_area.getText());
	}


	/**
	 * Returns a boolean array whose:
	 * <p> - first element indicates whether or not pixels that do meet the absolute
	 * RGB channel value thresholds are to be filtered
	 * <br> - second element indicates whether or not pixels that do not meet the RGB channel
	 * percentage thresholds are to be filtered
	 * <br> - third element indicates whether or not pixels that do not meet the average
	 * brightness thresholds are to be filtered
	 * <br> - fourth element indicates whether or not pixels that do meet the maximum
	 * vertical run length threshold are to be filtered
	 * <br> - fifth element indicates whether or not pixels that do meet the minimum
	 * horizontal run length are to be filtered
	 *
	 * <p>These settings are determined by the the <i>Filter Pixels By</i>  checkboxes.
	 *
	 * @return	The settings defined by the check boxes in this dialog box.
	 */
	public boolean[] getFiltersToUse()
	{
		boolean[] settings = new boolean[5];
		for (int i = 0; i < settings.length; i++)
			settings[i] = false;

		if (filter_by_colour_absolute_values_checkbox.isSelected())
			settings[0] = true;
		if (filter_by_colour_fractions_checkbox.isSelected())
			settings[1] = true;
		if (filter_by_brightness_checkbox.isSelected())
			settings[2] = true;
		if (filter_by_vertical_run_length_checkbox.isSelected())
			settings[3] = true;
		if (filter_by_horizontal_run_length_checkbox.isSelected())
			settings[4] = true;

		return settings;
	}


	/**
	 * Returns the maximum number of layers to display when the <i>Test Thresholds</i>
	 * button is pressed, as entered in the text field.
	 */
	public int getLayersToDisplay()
	{
		return Integer.parseInt(number_layers_to_display_text_area.getText());
	}


	/* PRIVATE METHODS *********************************************************/


	/**
	 * Restore text fields and check boxes to their defaults
	 */
	private void restoreDefaultValues()
	{
		red_detection_low_threshold_text_area.setText("160");
		red_detection_high_threshold_text_area.setText("255");
		green_detection_low_threshold_text_area.setText("0");
		green_detection_high_threshold_text_area.setText("153");
		blue_detection_low_threshold_text_area.setText("0");
		blue_detection_high_threshold_text_area.setText("140");
		red_min_percentage_threshold_text_area.setText("40.0");
		red_max_percentage_threshold_text_area.setText("100.0");
		green_min_percentage_threshold_text_area.setText("0.0");
		green_max_percentage_threshold_text_area.setText("33.0");
		blue_min_percentage_threshold_text_area.setText("0.0");
		blue_max_percentage_threshold_text_area.setText("28.0");
		low_brightness_threshold_text_area.setText("97.0");
		high_brightness_threshold_text_area.setText("255.0");
		vertical_run_length_threshold_text_area.setText("5");
		horizontal_run_length_threshold_text_area.setText("5");
		number_layers_to_display_text_area.setText("1");

		filter_by_colour_absolute_values_checkbox.setSelected(true);
		filter_by_colour_fractions_checkbox.setSelected(true);
		filter_by_brightness_checkbox.setSelected(false);
		filter_by_vertical_run_length_checkbox.setSelected(true);
		filter_by_horizontal_run_length_checkbox.setSelected(true);

	}

	/**
	 * Hides this dialog box.
	 */
	private void done()
	{
		this.hide();
	}
}