/*
 * @(#)MainFrame.java	1.0 08/15/04
 *
 * McGill Univarsity
 */

import java.io.File;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * A GUI frame that allows the user to display, load, save and modify image files.
 * This is intended primarily for processing image files that portray musical scores.
 *
 * <p>The <i>File</i> menu has the following options:
 *
 * <p><i>Load Image:</i> Loads and displays a new image file.
 * <br><i>Save As:</i> Saves the currently displayed image file under a user provided name.
 * <br><i>Quit:</i> Quits the program.
 *
 * <p>The <i>Process</i> menu has the following options:
 *
 * <p><i>Restore Image:</i> Returns the display to that of the image found in the file that
 * is currently referenced.
 * <br><i>Rotate:</i> Rotates the displayed image by a user specified amount.
 * <br><i>Remove Staff Lines:</i> Restores the image to that of the file that was last loaded
 * or saved and removes horizontal lines that fall within user specified colour and run length
 * thresholds. These settings are specified in the Horizontal Line Removal Settings Dialog Box.
 *
 * <p>The <i>Settings</i> menu has the following options:
 *
 * <p><i>Horizontal Line Removal Settings:</i> Displays the Horizontal Line Removal Settings
 * Dialog Box that allows the user to set the thresholds for the <i>Remove Staff Lines</i>
 * command and to highlight selected pixels on the image.
 *
 * <p>The button panel has buttons that duplicate commonly used menu commands.
 *
 * <p>The text area at the bottom of the window displays the currently loaded image file
 * and other messages.
 *
 * <p>Upon instantiation, objects of this class automatically open a specified file
 * or default to a file in the default directory called "example.jpg".
 *
 * <p>The <code>rotate</code> method can be called to rotate the displayed image based
 * on the contents of the Rotate Dialog Box. The <code>showSelectedPixels</code> method
 * can be called to restore the image to that of the file on disk and highlight
 * certain pixels based on the settings in the Horizontal Line Removal Dialog Box.
 * The <code>setNewFilePath</code> method can be called to store the name of a new file
 * to serve as the original of the currently displayed image and display it in the bottom
 * text area..
 *
 * @author Cory McKay
 */ 
public class MainFrame
	extends JFrame
	implements ActionListener
{
	/* FIELDS ******************************************************************/

	
	// Image fields
	private String									image_file_name;				// the path of the current file
	private ImagePanel								image_processor;				// the object used to store, display and process each image

	// GUI container fields
	private	Container								content_pane;					// hold all elements of the GUI
	private JTextArea								text_area;						// the area used to give the user feedback
	private JScrollPane								text_scroll_pane;				// the ScrollPane used to store textArea
	private JScrollPane								image_scroll_pane;				// the ScrollPane used to store image_processor

	// Dialog box fields
	private JFileChooser							load_file_chooser;				// a dialog box to open files
	private JFileChooser							save_file_chooser;				// a dialog box to save files
	private HorizontalLineRemovalSettingsDialogBox	staff_line_removal_dialog_box;	// settings for removing staff lines
	private RotateDialogBox							rotate_dialog_box;				// settings for rotating the image

	// Menu fields
	private JMenuBar								menu_bar;						// holds menu items
	private JMenu									file_menu;						// menu used to perform file operations
	private JMenu									process_image_menu;				// menu used to process images
	private JMenu									settings_menu;					// menu used to set image processing settings
	private JMenuItem								load_menu_item;					// menu item used to open a new file
	private JMenuItem								save_as_menu_item;				// menu item used to save the current image
	private JMenuItem								quit_menu_item;					// menu item used to quit the program
	private JMenuItem								restore_menu_item;				// menu item used to restore the currently loaded image to the state in its file
	private	JMenuItem								rotate_menu_item;				// menu item used to rotate the image
	private	JMenuItem								remove_staff_lines_menu_item;	// menu item used to automatically remove staff lines
	private	JMenuItem								staff_lines_settings_menu_item;	// menu item used to set remove staff lines settings

	// Button fields
	private JPanel									button_panel;					// holds buttons
	private JButton									load_button;					// button used to save the current image
	private JButton									restore_button;					// button used to restore the currently loaded image to the state in its file
	private JButton									staff_lines_settings_button;	// menu item used to set remove staff lines settings
	private JButton									remove_staff_lines_button;		// button used to automatically remove staff lines
	private JButton									save_as_button;					// button used to save the current image


	/* CONSTRUCTOR *************************************************************/


	/**
	 * Basic constructor that prepares, packs and shows all of the contents of the
	 * <code>JFrame</code>.
	 * 
	 * If a file path is specified to the <i>file_name</i> parameter, then this file
	 * will automatically be opened. If a null is given as the parameter, then a
	 * <code>JFileChooser</code> dialog box is shown on startup in order to select a
	 * file. If any problems occur, then the program automatically opens a file called
	 * "example.jpg" in the current directory.
	 *
	 * @param	file_name	The path of a file to display initially. May be null.
	 */
	public MainFrame(String file_name)
	{
		// Configure the title of the JFrame
		setTitle("ScoreReader");

		// Configure dialog boxes
		load_file_chooser = new JFileChooser();
		load_file_chooser.setCurrentDirectory(new File("."));
		save_file_chooser = new JFileChooser();
		save_file_chooser.setCurrentDirectory(new File("."));
		staff_line_removal_dialog_box = new HorizontalLineRemovalSettingsDialogBox(this);
		rotate_dialog_box = new RotateDialogBox(this);

		// Cause the program to quit when the exit box is pressed
		addWindowListener(new WindowAdapter() {
			public void windowClosing (WindowEvent e) {
				quit();
			}
		});

		// Display the text area where feedback to the user is given. Put it in a ScrollPane
		text_area = new JTextArea();
		text_area.setLineWrap(false);
		text_area.setRows(3);
		text_area.setColumns(45);
		text_area.setEditable(false);
		text_scroll_pane = new JScrollPane(text_area);
		text_area.setText("");

		// Load an image and display it in the image_processor ImagePanel in a ScrolPane
		// If a filename was specified by the calling object, then this file is opened.
		// If a filename was not specified, then a JFileChooser dialog box is shown on startup.
		// NOTE: the program will automatically open example.jpg if an invalid file is specified.
		if (file_name != null)
			image_file_name = file_name;
		else
		{
			int chooser_return;
			chooser_return = load_file_chooser.showOpenDialog(MainFrame.this);
			if (chooser_return != JFileChooser.APPROVE_OPTION)
				System.exit(0);
			image_file_name = load_file_chooser.getSelectedFile().getPath();
		}
		image_processor = new ImagePanel(image_file_name, this);
		image_scroll_pane = new JScrollPane();
		image_scroll_pane.setViewportView(image_processor);

		// Set up the File menu
		file_menu = new JMenu("File");
		load_menu_item = new JMenuItem("Load Image");
		file_menu.add(load_menu_item);
		load_menu_item.addActionListener(this);
		save_as_menu_item = new JMenuItem("Save As");
		file_menu.add(save_as_menu_item);
		save_as_menu_item.addActionListener(this);
		quit_menu_item = new JMenuItem("Quit");
		file_menu.add(quit_menu_item);
		quit_menu_item.addActionListener(this);
		
		// Set up the Process Image menu
		process_image_menu = new JMenu("Process");
		restore_menu_item = new JMenuItem("Restore Image");
		process_image_menu.add(restore_menu_item);
		restore_menu_item.addActionListener(this);
		rotate_menu_item = new JMenuItem("Rotate");
		process_image_menu.add(rotate_menu_item);
		rotate_menu_item.addActionListener(this);
		remove_staff_lines_menu_item = new JMenuItem("Remove Staff Lines");
		process_image_menu.add(remove_staff_lines_menu_item);
		remove_staff_lines_menu_item.addActionListener(this);

		// Set up the Settings Menu
		settings_menu = new JMenu("Settings");
		staff_lines_settings_menu_item = new JMenuItem("Horizontal Line Removal Settings");
		settings_menu.add(staff_lines_settings_menu_item);
		staff_lines_settings_menu_item.addActionListener(this);

		// Set up the menu bar
		menu_bar = new JMenuBar();
		setJMenuBar(menu_bar);
		menu_bar.add(file_menu);
		menu_bar.add(process_image_menu);
		menu_bar.add(settings_menu);

		// Set up the buttons
		load_button = new JButton("Load Image");
		load_button.addActionListener(this);
		restore_button = new JButton("Restore Image");
		restore_button.addActionListener(this);
		staff_lines_settings_button = new JButton("Staff Line Settings");
		staff_lines_settings_button.addActionListener(this); 
		remove_staff_lines_button = new JButton("Remove Staff Lines");
		remove_staff_lines_button.addActionListener(this);
		save_as_button = new JButton("Save As");
		save_as_button.addActionListener(this);

		// Set up the panel of buttons
		button_panel = new JPanel();
		button_panel.add(load_button);
		button_panel.add(restore_button);
		button_panel.add(staff_lines_settings_button);
		button_panel.add(remove_staff_lines_button);
		button_panel.add(save_as_button);

		// Display the image in the centre of the frame, the button panel
		// at the top of the frame and the text area at the bottom of the frame
		content_pane = getContentPane();
		content_pane.add(button_panel, BorderLayout.NORTH);
		content_pane.add(image_scroll_pane, BorderLayout.CENTER);
		content_pane.add(text_scroll_pane, BorderLayout.SOUTH);
	}


	/* PUBLIC METHODS **********************************************************/

	
	/**
	 * Calls the appropriate methods when the buttons are pressed or menu items selected.
	 *
	 * @param	event		The event that is to be reacted to.
	 */
	public void actionPerformed(ActionEvent event)
	{
		// React to the load_menu_item or load_button
		if ( event.getSource().equals(load_menu_item) ||
			 event.getSource().equals(load_button) )
			loadNewImage();

		// React to the save_as_menu_item or save_as_button
		else if ( event.getSource().equals(save_as_menu_item) ||
		          event.getSource().equals(save_as_button) )
			save_as();

		// React to the quit_menu_item
		else if (event.getSource().equals(quit_menu_item))
			quit();

		// React to the restore_menu_item
		else if (event.getSource().equals(restore_menu_item))
			restoreImage();

		// React to the restore_menu_item or the restore_button
		else if ( event.getSource().equals(restore_menu_item) ||
		          event.getSource().equals(restore_button) )
			restoreImage();

		// React to the rotate_menu_item
		else if ( event.getSource().equals(rotate_menu_item) )
			rotate_dialog_box.show();

		// React to the remove_staff_lines_menu_item or the remove_staff_lines_button
		else if ( event.getSource().equals(remove_staff_lines_menu_item) ||
		          event.getSource().equals(remove_staff_lines_button)) 
			removeHorizontalLines();

		// React to the staff_lines_settings_menu_item
		else if ( event.getSource().equals(staff_lines_settings_menu_item) ||
		          event.getSource().equals(staff_lines_settings_button) )
			staff_line_removal_dialog_box.show();
	}


	/**
	 * Rotate the displayed image based on the contents of the
	 * Rotate Dialog Box.
	 */
	public void rotate()
	{
		image_processor.rotate(rotate_dialog_box);
	}


	/**
	 * Restore the image to that of the file on disk and highlight
	 * certain pixels based on the settings in the Horizontal Line
	 * Removal Dialog Box.
	 */
	public void showSelectedPixels()
	{
		restoreImage();
		image_processor.showSelectedPixels(staff_line_removal_dialog_box);
	}


	/**
	 * Store the name of a new file to serve as the original of the
	 * currently displayed image and display it in the bottom text
	 * area.
	 *
	 * @param	file_path	The path of the new image file.
	 */
	public void setNewFilePath(String file_path)
	{
		image_file_name = file_path;
		text_area.setText(image_file_name);
	}


	/* PRIVATE METHODS *********************************************************/


	/**
	 * Brings up a <code>JFileChooser<code> dialog box to open new image files and
	 * displays the new file if a file is selected to be loaded.
	 *
	 * <p><b>NOTE:</b> will automatically open example.jpg if an invalid file is
	 * specified.
	 */
	private void loadNewImage()
	{
		int chooser_return = load_file_chooser.showOpenDialog(MainFrame.this);
		if (chooser_return == JFileChooser.APPROVE_OPTION) // only do if OK chosen
		{
			image_processor.refreshImage(load_file_chooser.getSelectedFile().getPath());
			MainFrame.this.image_scroll_pane.revalidate();
		}
	}


	/**
	 * Saves the image that is currently displayed to disk. Brings up a
	 * <code>JFileChooser<code> dialog box to allow the user to select an appropriate
	 * file name and location. Defaults the name to the name of the last loaded
	 * file name with "_proc" added to the end of its name. Saves the file as a
	 * JPEG.
	 *
	 * <p>If the selected file already exists, the user is asked if s/he wishes to
	 * overwrite it. Displays an informative error message if a problem is encountered.
	 */
	private void save_as()
	{
		// Add the default "_proc" to the new file name
		save_file_chooser.setSelectedFile(new File(removeExtension(image_file_name) + "_proc" + ".jpg"));
		
		// Use the JFileChooser to save the file.
		int save_return = save_file_chooser.showSaveDialog(this);
		if (save_return == JFileChooser.APPROVE_OPTION) // only do if OK chosen
		{
			File selection = save_file_chooser.getSelectedFile();
			boolean go_ahead = true;
			
			// Check to see if an existing file should be overwritten
			if (selection.exists())
			{
				int response = JOptionPane.showConfirmDialog(MainFrame.this, "A file " +
					"with the specified path already exists.\nDo you wish to overwrite it?", "Warning",
					JOptionPane.OK_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE);
				if (response != JOptionPane.YES_OPTION)
					go_ahead = false;
			}
			
			// PErform the save
			if (go_ahead)
			{
				try
				{
					image_processor.saveImage(selection);

					// Update the file path in the parent_window and its text display
					setNewFilePath(selection.getPath());
				}
				catch (Exception e) { JOptionPane.showMessageDialog(null, ("Image could not be saved.\n" + e.getMessage()), "ERROR", JOptionPane.ERROR_MESSAGE); }
			}
		}
	}

	
	/**
	 * Exit the program.
	 */
	private void quit()
	{
		System.exit(0);
	}

	
	/**
	 * Restore the displayed image to that of the file that was last loaded or
	 * saved.
	 */
	private void restoreImage()
	{
		text_area.setText("");
		image_processor.refreshImage(image_file_name);
	}


	/**
	 * Restore the displayed image to that of the file that was last loaded or
	 * saved and remove vertical lines based on the settings in the Horizontal
	 * Line Removal Settings Dialog Box.
	 */
	private void removeHorizontalLines()
	{
		restoreImage();
		image_processor.removeHorizontalLines(staff_line_removal_dialog_box);
	}


	/**
	 * Returns a copy of the given string with the last four characters removed. Returns null
	 * if the fourth to last character is not a period or there are less than five characters
	 * in the string.
	 *
	 * @param	filename The name of the file from which the extension is to be removed.
	 * @return	The name of the file with the extension removed.
	 */
	private String removeExtension(String filename)
	{
		if (filename.length() < 5)
			return null;
		if (filename.charAt(filename.length() - 4) != '.')
			return null;
		return filename.substring(0, (filename.length() - 4));
	}
}